# frozen_string_literal: true

require 'rest-client'
require 'active_support'
require 'json'
require 'erb'

module Rancher
  module ApiDiscovery
    class Crawler
      def initialize
        @base_url = ENV['RANCHER_URL']
        @entities = {}
      end

      def run
        schemas = get(@base_url + '/schemas')
        schemas['data'].each do |entity|
          @entities[entity['id']] = Entity.new(entity)
        end
        @entities.each do |key, entity|
          puts "Discovered: #{key}"
          puts "helpers: #{entity.helpers}"
          puts "has_many: #{entity.has_many}"
          puts "has_one: #{entity.has_one}"
          puts "belongs_to: #{entity.belongs_to}"
          puts "attributes: #{entity.attributes}"
          puts "Generated code:\n"
          puts entity.code
          puts "\n"
        end
      end

      def generate_code
        @entities.each do |key, entity|
          filename = "./lib/rancher/api/models/#{key.downcase}.rb"
          File.write(filename, entity.code)
        end
        File.open('./lib/rancher/api/models.rb', 'w') do |file|
          @entities.each do |key, _|
            require_string = "require 'rancher/api/models/#{key.downcase}'"
            file.puts(require_string)
          end
        end
      end

      private

      def parse_response(json)
        JSON.parse(json)
      end

      def get(url)
        parse_response(RestClient.get(url))
      end
    end

    class Entity < OpenStruct
      def has_many
        includeableLinks.map(&:downcase).reject { |k| singular?(k) }
      end

      def has_one
        includeableLinks.map(&:downcase).select { |k| singular?(k) }
      end

      def belongs_to
        attributes.select { |k| k.end_with?('Id') }.reject { |k| k.eql?('externalId') }.map do |relation_key|
          relation_key.gsub('Id', '').downcase
        end
      end

      def attributes
        resourceFields.keys
      end

      def code
        renderer = ERB.new(File.read('./templates/model.erb'), nil, '-')
        renderer.result(binding)
      end

      def helpers
        return [] unless File.exist?(helper_filename)
        ["Rancher::Api::Helpers::#{id.capitalize}"]
      end

      def helper_filename
        "./lib/rancher/api/helpers/#{id}.rb"
      end

      private

      def singular?(str)
        str.singularize == str
      end
    end
  end
end
